#!/usr/bin/perl
################################################################################
# eWLA (e Web Log Analyzer)                         Version 0.1                #
# Copyright 2000 Frederic TYNDIUK (FTLS)            All Rights Reserved.       #
# E-Mail: tyndiuk@ftls.org                          Script License: GPL        #
# Created  05/31/00                                 Last Modified 05/25/00     #
# Scripts Archive at:                               http://www.ftls.org/cgi/   #
################################################################################
# Function : Creates multi-page HTML reports of Apache log statistics.         #
#   Graph part.                                                                #
################################################################################
########################## license & copyright header ##########################
#                                                                              #
#                     Copyright (c) 2000 by TYNDIUK Frederic                   #
#                                                                              #
#       This program is free software; you can redistribute it and/or          #
#       modify it under the terms of the GNU General Public License as         #
#       published by the Free Software Foundation; either version 2 of         #
#       the License, or (at your option) any later version.                    #
#                                                                              #
#       This program is distributed in the hope that it will be useful,        #
#       but WITHOUT ANY WARRANTY; without even the implied warranty of         #
#       MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          #
#       GNU General Public License for more details.                           #
#                                                                              #
#       You should have received a copy of the GNU General Public License      #
#       along with this program in the file 'COPYING'; if not, write to        #
#       the Free Software Foundation, Inc., 59 Temple Place - Suite 330,       #
#       Boston, MA 02111-1307, USA, or contact the author:                     #
#                                                                              #
#                                   TYNDIUK Frederic <tyndiuk@ftls.org>        #
#                                            <http://www.ftls.org/>            #
#                                                                              #
######################## end license & copyright header ########################
################################################################################
# Optionals  Variables:                                                        #

#use strict;
require 'ewla_lib.pl';

my %params;
$params{'ConfigFile'} = "ewla.cfg";

$params{'Look'} = "";
# Nothing Below this line needs to be altered!                                 #
################################################################################
#    DO NOT EDIT ANYTHING BELOW THIS LINE UNLESS YOU KNOW WHAT YOU'RE DOING    #
################################################################################
my $COPYRIGHT  = "eWLA v0.1 (C) 2000 Frederic TYNDIUK (alias FTLS)\n";
   $COPYRIGHT .= "Report bugs to tyndiuk\@ftls.org, News and Updates: http://www.ftls.org/\n\n";

my $VERBOSE = 1;
my $DEBUG = 0;
my $USE_GRAPH = 1;
################################################################################

if ($USE_GRAPH) {
	require GD::Graph::bars3d;
	require GD::Graph::lines3d;
	require GD::Graph::pie3d;
}

# Global variables :
my (%add_to_menu, %menu_data);
my (%trace_visit, %test_visit, %empty);
my (%all_hits_year, %all_bytes_year, %all_pages_views_year, %all_visits_year);
my (%all_hits_monthly, %all_bytes_monthly, %all_pages_views_monthly, %all_visits_monthly);
my (%all_hits_daily, %all_bytes_daily, %all_pages_views_daily, %all_visits_daily);
my (%all_hits_hourly, %all_bytes_hourly, %all_pages_views_hourly, %all_visits_hourly);
my (%all_hits_dayofweek, %all_bytes_dayofweek, %all_pages_views_dayofweek, %all_visits_dayofweek);
my (%files_hits_daily, %files_bytes_daily, %files_visits_daily, %files_pages_views_daily);
my (%domains_hits_daily, %domains_bytes_daily, %domains_visits_daily, %domains_pages_views_daily);
my (%users_hits_daily, %users_bytes_daily, %users_visits_daily, %users_pages_views_daily);
my (%referrer_hits_daily, %referrer_visits_daily, %referrer_pages_views_daily);
my (%agent_hits_daily, %agent_visits_daily, %agent_pages_views_daily);
my (%files_error_daily, %seeker, %record, %language, %tld_contries, %adverts_hits_daily);
my (%template_infos);

#Other variables :
	my ($resumes_dir, $reports_dir, $language_file, @lang, @info, @tld);


########################################
# Read args and config file and Init   #
########################################

%params = &read_args(%params);

%params = &load_data_from_config_file(%params);

$params{'ServerURL'} =~ s/\/$//;
$template_infos{'site_url'}  = $params{'ServerURL'};
$template_infos{'site_name'} = $params{'ServerName'};
$resumes_dir =  $params{'ResumeDir'};
$reports_dir =  $params{'OutputDir'};

print "Loading language file\n" if ($DEBUG);
@lang = &load_txt_file($params{'LangFile'}, 'language');
my $i = "00";
foreach(@lang) {
	if ((! /^#/) && (/ : /)) {
		chomp($_);
		s/\s+$//;
		@info = split (/\s*:\s*/, $_, 2);
		if (/^tld_/) {
			$info[0] =~ s/tld_//;
			$tld_contries{$info[0]} = $info[1]." (.$info[0])";
		} elsif (/^menu_(\d)_*(.*)$/) {
			my $no = $1;
			my ($m, $t) = split(/\s*:\s*/, $2);
			if ($m ne "")  {
				$i++;
				$menu_data{$m} = "$no|$i|$t";
			} else {
				$menu_data{$no} = $t;
				$i = "00";
			}
		} else {
			s/^\s*//g;
			$language{$info[0]} = $info[1];
		}
	}
}

# Init vars
my @Week_days         = split(/\s*,\s*/, $language{'word_wdays'});
my @Week_days_short   = split(/\s*,\s*/, $language{'word_wdays_short'});
my @Months       = split(/\s*,\s*/, $language{'word_months'});
my @Months_short = split(/\s*,\s*/, $language{'word_months_short'});

$params{'DailyReports'} = ! $params{'NoDailyReports'};
$template_infos{'gene_date'} = &conv_date('', "%e %B %Y");
my $html_copyright = &loadcopyright();

########################################
# Begin to create Report               #
########################################
print $COPYRIGHT;
print "Report Creator for $params{'ServerName'}\n" if ($VERBOSE);

my $mk_all_reports = 0;
foreach(<$resumes_dir*>) {
	my $resume_file = $_;
	$resume_file =~ s/$resumes_dir//;
	if (/([mdy])(\d\d\d\d)_*(\d\d)*_*(\d\d)*/) {
		my ($type, $year, $month, $day) = ($1, $2, $3, $4);
		if (! -d "$reports_dir$year/") { mkdir ("$reports_dir$year/", 0755); }
		if (! -d "$reports_dir$year/$month/") { mkdir ("$reports_dir$year/$month/", 0755); }
		if ((! -d "$reports_dir$year/$month/$day/") && $params{'DailyReports'})  { mkdir ("$reports_dir$year/$month/$day/", 0755); }
	}
}

foreach(<$resumes_dir*>) {
	my $resume_file = $_;
	$resume_file =~ s/$resumes_dir//;
	%add_to_menu = ();
	if (/([mdy])(\d\d\d\d)_*(\d\d)*_*(\d\d)*/) {
		my ($type, $year, $month, $day) = ($1, $2, $3, $4);

		$template_infos{'scale_date_day'} = $day;
		$template_infos{'scale_date_mth'} = $Months[$month -1];
		$template_infos{'scale_date_yer'} = $year;
		
		if (($type eq 'd') && $params{'DailyReports'}) { 
			$template_infos{'scale_date'} = "$language{'word_daily_report'} : $day $Months[$month -1] $year"; 
			$template_infos{'scale_date_report_type'} = $language{'word_daily_report'}; 
			$template_infos{'scale_date_report_time'} = "$day $Months[$month -1] $year";
			$template_infos{'img_up_dir'} = '../../..';
			$template_infos{'cal_up_dir'} = '../..';
		}
		if ($type eq 'm') { 
			$template_infos{'scale_date'} = "$language{'word_monthly_report'} : $Months[$month -1] $year"; 
			$template_infos{'scale_date_report_type'} = $language{'word_monthly_report'}; 
			$template_infos{'scale_date_report_time'} = "$Months[$month -1] $year";
			$template_infos{'img_up_dir'} = '../..';
			$template_infos{'cal_up_dir'} = '..';
			$template_infos{'scale_date_day'} = "";
		}
		if ($type eq 'y') { 
			$template_infos{'scale_date'} = "$language{'word_yearly_report'} $year"; 
			$template_infos{'scale_date_report_type'} = $language{'word_yearly_report'}; 
			$template_infos{'scale_date_report_time'} = $year;
			$template_infos{'img_up_dir'} = '..';
			$template_infos{'cal_up_dir'} = '.';
			$template_infos{'scale_date_day'} = $template_infos{'scale_date_mth'} = "";
		}

		if (! (($type eq 'd') && $params{'NoDailyReports'})) { 
		if ($mk_all_reports || (&last_updated_file($resumes_dir.$resume_file) > &last_updated_file("$reports_dir$year/$month/$day/index.html"))) {
			&mk_daily_report("$reports_dir$year/$month/$day/", $type, $resumes_dir.$resume_file, $day, $month, $year);
			&mk_index($reports_dir, $type, $day, $month, $year);
			}
		}
	}
}

# End

########################################
# Sub fonction                         #
########################################


sub mk_index {
	my($report_dir_base, $report_type, $day, $month, $year) = @_;
	my ($tmp, $tmp2, $report_dir, $up, $i_txt, $month_name, $cal);
	$report_dir = $report_dir_base."$year/$month/$day/";
	$report_dir =~ s/\/+/\//g;
	$up = $template_infos{'cal_up_dir'};
	$template_infos{'calendar'} = '';

	# Make Calendar	
	$template_infos{'cgi_cal_param'} = "y=$year&m=$month&d=$day&t=$report_type&tytxt=$template_infos{'scale_date_report_type'}";
	$template_infos{'cgi_cal_param'} =~ s/ /%20/g;
	if ($report_type ne 'y') {
		$cal = &calendar($day, $month, $year, $report_dir_base, $report_type);
	} else {
		$cal = "&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<B>$year</B>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;";
	}
	if ($params{'NoDailyReports'}) {
		$cal = "&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<B>$year</B>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;";
	}

	$template_infos{'calendar'} = "<TABLE BORDER=\"0\"><TR><TD VALIGN=\"top\"><FONT FACE=\"Arial\" SIZE=\"1\" COLOR=\"$params{'CalMonthTxtColor'}\">";
	
	for (my $i = 1; $i <= 12; $i++) {
		if ($i < 10) { $i_txt = "0$i" } else  { $i_txt = $i; }
		$month_name = $Months_short[$i -1];
		if (-d $report_dir_base."$year/$i_txt/")  { 
			$month_name = "<A HREF=\"$up/$i_txt/index.html\" TARGET=\"_top\">$month_name</A>"
		}
		if ($i == 7) { $template_infos{'calendar'} .= "</FONT></TD><TD ALIGN=\"CENTER\">\n$cal\n</TD><TD VALIGN=\"top\"><FONT FACE=\"Arial\" SIZE=\"1\" COLOR=\"$params{'CalMonthTxtColor'}\">"; }
		$template_infos{'calendar'} .= "$month_name<BR>";
	}
	$template_infos{'calendar'} .= "</FONT></TD></TR></TABLE>";

	# Make Menu

	my (%lst, $html_menu);
	# Make menu
	foreach(keys (%add_to_menu)) {
		my ($no, $i, $txt) = split(/\|/, $menu_data{$_});
		$lst{"$no|$menu_data{$no}"} .= "$i\t$_\t$txt\n";
	}

	$html_menu = "";
	foreach (sort (keys %lst)) {
		my $m_title = $_;
		$m_title =~ s/\d+\|//;
		my $pres = $params{'MenuSectBegin'};
		$pres =~ s/Value_txt/$m_title/ig;
		$html_menu .= $pres."\n";
		foreach $_ (sort split(/\n/, $lst{$_})) {
			s/\d+\t//;
			my ($file, $txt) = split(/\t/, $_);
			my $pres = $params{'MenuItem'};
			$pres =~ s/Value_link/$file.html/ig;
			$pres =~ s/Value_txt/$txt/ig;
			$html_menu .= $pres."\n";
		}
		$html_menu .= $params{'MenuSectEnd'};
	}
	$html_menu =~ s/Value_img_up_dir/$template_infos{'img_up_dir'}/ig;
	$template_infos{'menu'} = $html_menu;


	foreach(split (/\s*,\s*/, $params{'TemplateOther'})) {
		if (/\w/) {
			/([^\/]+$)/;
			my $name = $1;
			my $tmp = &repace_into_template_file($_, %template_infos);
			&save_txt_file($report_dir.$name, 'index', $tmp);
		}
	}
}

sub mk_daily_report {
	my($report_dir, $report_type, $resume_file, $day, $month, $year) = @_;
	$report_dir =~ s/\/+/\//g;
	&init_resume_value();
	&load_resume_file($resume_file);
	print "      Make HTML report for $day/$month/$year (dir: $report_dir)\n" if ($VERBOSE);

	if ($params{'HourlyReport'}) {
	&mk_day_type_report('hourly', "$report_dir", 
		\%all_hits_hourly, \%all_bytes_hourly, \%all_visits_hourly, \%all_pages_views_hourly);
	}

	# Daily
	if ($report_type ne 'd') {
		$all_hits_daily{'00'} = '';
		if ($params{'DailyReport'}) {
			&mk_day_type_report('daily', "$report_dir", 
				\%all_hits_daily, \%all_bytes_daily, \%all_visits_daily, \%all_pages_views_daily);
		}
		if ($params{'DayOfWeekReport'}) {
			&mk_day_type_report('dayofweek', "$report_dir", 
				\%all_hits_dayofweek, \%all_bytes_dayofweek, \%all_visits_dayofweek, \%all_pages_views_dayofweek);
		}
	}

	# monthly
	if (($report_type eq 'y') && $params{'MonthlyReport'}) {
		$all_hits_monthly{'00'} = '';
		&mk_day_type_report('monthly', "$report_dir",
			\%all_hits_monthly, \%all_bytes_monthly, \%all_visits_monthly, \%all_pages_views_monthly);
	}

	&mk_topoffiles_report($report_dir);
	&mk_org_contries_report($report_dir);

	if ($params{'ReferrersReport'}) { &mk_referrers_report($report_dir); }
	if ($params{'SeekerReport'})   { &mk_seeker_report($report_dir); }
	if ($params{'AgentsReport'})   { &mk_agents_report($report_dir); }
	if ($params{'ErrorsReport'} && ($report_type ne 'd'))   { 
		&mk_errors_report($report_dir); 
	}

	&mk_paths_report($report_dir);
	
	my ($total_hits, $total_bytes, $total_visits, $total_pv) = &mk_intro_report($report_dir, $report_type, \%all_hits_hourly, \%all_bytes_hourly, \%all_visits_hourly, \%all_pages_views_hourly);

	if ($params{'AuthUsersReport'} && ((keys %users_hits_daily) > 1)) {
		my $total_hits_u;
		foreach (%users_hits_daily) { $total_hits_u += $users_hits_daily{$_}; }
		&mk_file_type_report('authusers', $report_dir, $params{'TopOfAuthUserT'} , $params{'TopOfAuthUserG'}, 'n authusers',
			\%users_hits_daily, \%users_bytes_daily, 
			\%users_visits_daily, \%users_pages_views_daily, $total_hits_u);
	}

} # end &mk_daily_report()

########################################
# Sub for each report themes.          #
########################################

sub mk_intro_report {
	my ($report_dir, $report_type, $hits, $bytes, $visits, $pv) = @_;
	my $id = 'intro';
	my ($nb, $total_hits, $total_bytes, $total_visits, $total_pv,
		$max_hits, $max_bytes, $max_visits, $max_pv, $val, $date,
		$avr_hits, $avr_bytes, $avr_visits, $avr_pv, %infos);

	foreach(keys %$hits) {
		$nb++;
		$total_hits += $$hits{$_};
		$total_bytes += $$bytes{$_};
		$total_visits += $$visits{$_};
		$total_pv += $$pv{$_};
		if ($$hits{$_} > $max_hits)    { $max_hits = $$hits{$_} }
		if ($$bytes{$_} > $max_bytes)  { $max_bytes = $$bytes{$_} }
		if ($$visits{$_} > $max_visits){ $max_visits = $$visits{$_} }
		if ($$pv{$_} > $max_pv)        { $max_pv = $$pv{$_} }
	}
	$avr_hits  = int($total_hits / $nb);
	$avr_bytes = int($total_bytes / $nb);
	$avr_visits= int($total_visits / $nb);
	$avr_pv    = int($total_pv / $nb);
	$total_bytes = int ($total_bytes / 1024);
	$avr_bytes = int ($avr_bytes / 1024);
	$max_bytes = int ($max_bytes  / 1024);

	$infos{"1|$language{'intro_txt_hits'}"}  = "$language{'intro_txt_total'}\t$total_hits\n";
	$infos{"1|$language{'intro_txt_hits'}"} .= "$language{'intro_txt_avr'}\t$avr_hits\n";
	$infos{"1|$language{'intro_txt_hits'}"} .= "$language{'intro_txt_home'}\t$files_hits_daily{'/'}\n";
	$infos{"1|$language{'intro_txt_hits'}"} .= "$language{'intro_txt_max'}\t$max_hits\n";
	$infos{"2|$language{'intro_txt_bytes'}"}  = "$language{'intro_txt_total'}\t$total_bytes\n";
	$infos{"2|$language{'intro_txt_bytes'}"} .= "$language{'intro_txt_avr'}\t$avr_bytes\n";
	$infos{"2|$language{'intro_txt_bytes'}"} .= "$language{'intro_txt_home'}\t".int($files_bytes_daily{'/'} / 1024)."\n";
	$infos{"2|$language{'intro_txt_bytes'}"} .= "$language{'intro_txt_max'}\t$max_bytes\n";
	$infos{"3|$language{'intro_txt_pv'}"}  = "$language{'intro_txt_total'}\t$total_pv\n";
	$infos{"3|$language{'intro_txt_pv'}"} .= "$language{'intro_txt_avr'}\t$avr_pv\n";
	$infos{"3|$language{'intro_txt_pv'}"} .= "$language{'intro_txt_home'}\t$files_hits_daily{'/'}\n";
	$infos{"3|$language{'intro_txt_pv'}"} .= "$language{'intro_txt_max'}\t$max_pv\n";
	$infos{"4|$language{'intro_txt_visits'}"}  = "$language{'intro_txt_total'}\t<B>$total_visits</B>\n";
	$infos{"4|$language{'intro_txt_visits'}"} .= "$language{'intro_txt_avr'}\t<B>$avr_visits</B>\n";
	$infos{"4|$language{'intro_txt_visits'}"} .= "$language{'intro_txt_home'}\t$files_visits_daily{'/'}\n";
	$infos{"4|$language{'intro_txt_visits'}"} .= "$language{'intro_txt_max'}\t$max_visits\n";

	if ($report_type ne 'd') {
		($val, $date) = split(/\t+/, $record{'hits'});
		$infos{"1|$language{'intro_txt_hits'}"} .= "$language{'intro_txt_record'} ($date)\t$val\n";
		($val, $date) = split(/\t/,$record{'bytes'});
		$val = int ($val / 1024);
		$infos{"2|$language{'intro_txt_bytes'}"}.= "$language{'intro_txt_record'} ($date)\t$val\n";
		($val, $date) = split(/\t/,$record{'pgv'});
		$infos{"3|$language{'intro_txt_pv'}"}    .= "$language{'intro_txt_record'} ($date)\t$val\n";
		($val, $date) = split(/\t/,$record{'visits'});
		$infos{"4|$language{'intro_txt_visits'}"} .= "$language{'intro_txt_record'} ($date)\t$val\n";
	}
	
	&mk_hash_sinple_report($id, $report_dir, '', \%infos);
	return ($total_hits, $total_bytes, $total_visits, $total_pv);
}

sub mk_paths_report {
	my ($report_dir) = @_;
	my ($total_hits, $total_sinle_hits, $t, $single);
	my (%tv, %top_exit_hits, %top_exit_bytes,
		%top_single_hits, %top_single_bytes);

	if (! ($params{'TopPathsReport'} 
		&& $params{'TopOfExitPagesReport'} 
		&& $params{'TopOfSinglePageReport'})) { return; }

	foreach (keys %trace_visit) {
		$t = $_;
		$t =~ s/^\t//;
		$t =~ s/\t+/<BR>/g;
		$tv{$t} = $trace_visit{$_};
		/\t([^\t]+)$/;
		my $exit = $1;
		if ($exit ne '') {
			$top_exit_hits{$exit} = $files_hits_daily{$exit};
			$top_exit_bytes{$exit} = $files_bytes_daily{$exit};
			$total_hits += $files_hits_daily{$exit};
		}
		if (/^\t([^\t]+)$/) {
			$single = $1;
			$top_single_hits{$single} = $files_hits_daily{$single};
			$top_single_bytes{$single} = $files_bytes_daily{$single};
			$total_sinle_hits += $files_hits_daily{$single};
		}
	}

	if ($params{'TopPathsReport'}) {
		&mk_file_type_report('topofpaths', $report_dir, $params{'TopOfFilesT'} , $params{'TopOfFilesG'}, 'n, paths',
				\%tv, '', '', '', 1);
	}

	if ($params{'TopOfExitPagesReport'}) {
		# Top of exit pages # no a
		&mk_file_type_report('topoffiles-exp', $report_dir, $params{'TopOfFilesT'} , $params{'TopOfFilesG'},  'n',
			\%top_exit_hits, \%top_exit_bytes, '', '', $total_hits);
	}

	if ($params{'TopOfSinglePageReport'}) {
		# Top of exit pages # no a
		&mk_file_type_report('topoffiles-sp', $report_dir, $params{'TopOfFilesT'} , $params{'TopOfFilesG'}, 'n',
			\%top_single_hits, \%top_single_bytes, '', '', $total_sinle_hits);
	}

}

sub mk_topoffiles_report {
	# Top of files
	my ($report_dir) =@_;
	my ($total_hits, $total_hits_download, $total_hits_cgi, $total_hits_dir, $ext, $dir, $fname);
	my (%files_dw_hits_daily, %files_dw_bytes_daily, %files_dw_visits_daily,
		%files_ext_hits_daily, %files_ext_bytes_daily, %files_ext_visits_daily,
		%files_cgi_hits_daily, %files_cgi_bytes_daily, %files_cgi_visits_daily,
		%files_dir_hits_daily, %files_dir_bytes_daily, %files_dir_visits_daily);

	my $download_ext = '(tar|gz|bzip2|zip|exe|pdf|rpm|deb)';
	my $download_cgi = '(cgi|pl|php3*|phtml|asp)';

	# calcul total for %
	$total_hits = 0;
	foreach (%files_hits_daily) { $total_hits += $files_hits_daily{$_}; }

	if ($params{'TopOfFilesReport'}) {
		# Top of files # no a
		&mk_file_type_report('topoffiles', $report_dir, $params{'TopOfFilesT'} , $params{'TopOfFilesG'}, 'n, l',
			\%files_hits_daily, \%files_bytes_daily, 
			\%files_visits_daily, \%files_pages_views_daily,  $total_hits);
	}
	if ($params{'TopOfEntryPagesReport'}) {
		# Top of entry pages # no a
		&mk_file_type_report('topoffiles-ep', "$report_dir", $params{'TopOfFilesT'} , $params{'TopOfFilesG'}, 'n, l, noKo',
			\%files_visits_daily, \%files_hits_daily, \%files_bytes_daily, \%files_pages_views_daily, $total_hits);
	}

	if (! $params{'TopOfDwAndTypes'}) {
		return;
	}

	foreach(keys %files_hits_daily) {
		if (/\.$download_cgi$/) {
			$files_cgi_hits_daily{$_}   += $files_hits_daily{$_};
			$files_cgi_bytes_daily{$_}  += $files_bytes_daily{$_};
			$files_cgi_visits_daily{$_} += $files_visits_daily{$_};
			$total_hits_cgi += $files_hits_daily{$_}; 
			}
		if (/\.$download_ext$/) {
			/\/([^\/]+.$download_ext)$/;
			$fname = $1;
			$files_dw_hits_daily{$fname}   += $files_hits_daily{$_};
			$files_dw_bytes_daily{$fname}  += $files_bytes_daily{$_};
			$files_dw_visits_daily{$fname} += $files_visits_daily{$_};
			$total_hits_download += $files_hits_daily{$_}; 
			}
		if ((/\.([^\.]+)$/) || (/(.*Images Files)/)) {
			$ext = $1;
			$ext =~ s/\)+//;
			if ((length $ext >= 3) && (length $ext <= 6)){
				$files_ext_hits_daily{$ext}   += $files_hits_daily{$_};
				$files_ext_bytes_daily{$ext}  += $files_bytes_daily{$_};
				$files_ext_visits_daily{$ext} += $files_visits_daily{$_};
			}
		}
		$dir = $_;
		$dir =~ s/[^\/]+$//;
		if ($dir ne '') {
			$files_dir_hits_daily{$dir}   += $files_hits_daily{$_};
			$files_dir_bytes_daily{$dir}  += $files_bytes_daily{$_};
			$files_dir_visits_daily{$dir} += $files_visits_daily{$_};
			$total_hits_dir += $files_hits_daily{$_}; 			
		}
	} # end foreach

	# Top of download files # no a
	if ($total_hits_download > 1) {
		&mk_file_type_report('topoffiles-dw', $report_dir, $params{'TopOfFilesT'} , $params{'TopOfFilesG'}, 'n',
			\%files_dw_hits_daily, \%files_dw_bytes_daily, 
			\%files_dw_visits_daily, '', $total_hits_download);
	}

	# Top of download extens files type
	&mk_file_type_report('topoffiles-dw-type', $report_dir, $params{'TopOfFilesT'} , $params{'TopOfFilesG'}, 'n',
		\%files_ext_hits_daily, \%files_ext_bytes_daily, 
		\%files_ext_visits_daily, '', $total_hits);

	# Top of cgi
	if ($total_hits_cgi > 1) {
		&mk_file_type_report('topoffiles-cgi', $report_dir, $params{'TopOfFilesT'} , $params{'TopOfFilesG'}, 'n',
			\%files_cgi_hits_daily, \%files_cgi_bytes_daily, 
			\%files_cgi_visits_daily, '', $total_hits_cgi);
	}

	# Top of dir # no a
	&mk_file_type_report('topoffiles-dir', $report_dir, $params{'TopOfFilesT'} , $params{'TopOfFilesG'}, 'n',
		\%files_dir_hits_daily, \%files_dir_bytes_daily, 
		\%files_dir_visits_daily, '', $total_hits_dir);
}


sub mk_org_contries_report {
	# Most Active Organisation / Countries
	my($report_dir) = @_;
	my($ext, $total_hits);
	my(%contries_hits_daily, %contries_bytes_daily, %contries_pages_views_daily, %contries_visits_daily);

	foreach( keys %domains_hits_daily) { 
		$total_hits += $domains_hits_daily{$_}; 
		if (/\.([^\.]+)$/) {
			$ext = $tld_contries{$1};
			$contries_hits_daily{$ext} += $domains_hits_daily{$_};
			$contries_bytes_daily{$ext} += $domains_bytes_daily{$_};
			$contries_pages_views_daily{$ext} += $domains_pages_views_daily{$_};
			$contries_visits_daily{$ext} += $domains_visits_daily{$_};
		}
	}
	# no a
	if ($params{'OrgReport'}) {
		&mk_file_type_report('topoforg', $report_dir, $params{'TopOfOrgCtryT'} , $params{'TopOfOrgCtryG'}, 'n countrie',
			\%domains_hits_daily, \%domains_bytes_daily, 
			\%domains_pages_views_daily, \%domains_visits_daily, $total_hits);
	}
	
	# no a
	if ($params{'CountriesReport'}) {
		&mk_file_type_report('topofcountries', $report_dir, $params{'TopOfOrgCtryT'} , $params{'TopOfOrgCtryG'}, 'n countrie',
			\%contries_hits_daily, \%contries_bytes_daily, 
			\%contries_pages_views_daily, \%contries_visits_daily, $total_hits);
	}
} # end &mk_org_contries_report()


sub mk_referrers_report {
	my ($report_dir) = @_;
	my ($dir, $url, @a, $total_hits);
	my (%referrers, %referrers_domain1, %referrers_domain2);

	if (! ($params{'MinRefsHits'} >= 1))  { $params{'MinRefsHits'} = 1; }

	foreach (sort(keys %referrer_hits_daily)) {
		($dir, $url) = split(/\|/, $_);
		if ($referrer_hits_daily{$_} >= $params{'MinRefsHits'}) {
			$referrers{$dir} .= "$url\t$referrer_hits_daily{$_}\n";
		}
		$url =~ s/http:\/\///;
		$url =~ s/\/.*//g;
		$url =~ /([\w\-]+)\.(\w+)$/;
		$url = "http://www.$1.$2/";
		$referrers_domain1{"$dir|$url"} += $referrer_hits_daily{$_};
		$referrers_domain2{$url} += $referrer_hits_daily{$_};
		$total_hits += $referrer_hits_daily{$_};
	}
	foreach (sort(keys %referrers)) {
		@a = split(/\n/, $referrers{$_}); 
		@a = reverse (sort SubSortTxtByCol @a);
		$referrers{$_} = join("\n", @a);
	}
	&mk_hash_sinple_report('referrers-all', $report_dir, '', \%referrers, '', 'referrers.gif');

	%referrers = ();
	foreach (sort(keys %referrers_domain1)) {
		($dir, $url) = split(/\|/, $_);
		if ($referrers_domain1{$_} >= $params{'MinRefsHits'}) {
			$referrers{$dir} .= "$url\t$referrers_domain1{$_}\n";
		}
	}
	foreach (sort(keys %referrers)) {
		@a = split(/\n/, $referrers{$_}); 
		@a = reverse (sort SubSortTxtByCol @a);
		$referrers{$_} = join("\n", @a);
	}
	&mk_hash_sinple_report('referrers-d', $report_dir, '', \%referrers, '', 'referrers.gif');

	&mk_file_type_report('referrers', $report_dir, $params{'TopOfRefsT'}, $params{'TopOfRefsG'}, 'n, refs',
		\%referrers_domain2, '', '', '', $total_hits);

} # end &mk_referrers_report();

sub mk_agents_report {
	my ($report_dir) = @_;
	my (@data_col0, @data_col1, @data_col2, @data_col3, @data_col4);
	my ($i, $v, $total_hits, $os_type, $agt_type, $os_type_long, $os_type_short, $agt_type_long, $agt_type_short);
	my (%agent_type_long_hits, %agent_type_long_pview, %agent_type_long_visits,
		%agent_type_hits, %agent_type_pview, %agent_type_visits,
		%os_type_hits, %os_type_pview, %os_type_visits);

	if (! ($params{'MinAgentHits'} >= 1))  { $params{'MinAgentHits'} = 1; }
	
# Agents
	# Attentions graphs 5... + Pie...

	# All
	foreach (sort(keys %agent_hits_daily)) {
		if ($agent_hits_daily{$_} > $params{'MinAgentHits'}) {
			$data_col0[$i] = $_;
			$data_col1[$i] = $agent_hits_daily{$_};
			$data_col2[$i] = "";
			$data_col3[$i] = $agent_visits_daily{$_};
			$data_col4[$i] = $agent_pages_views_daily{$_};
			$total_hits += $agent_hits_daily{$_};
			$i++;
		}

			if (/Win(dows *)* *([\dNT\.]+)/i) { $os_type = "Windows $2"; }
			elsif (/mac/i) { 
				if (/p(ower)*pc/i) { $os_type = "Mac PowerPC" } else { $os_type = "Macintosh 68K"; } }
			elsif (/amiga/i) { $os_type = "Amiga"; }
			elsif (/os\/2/i) { $os_type = "OS/2"; }
			elsif (/x11.*[^a-z](linux|hp-ux|sunos|freebsd|netbsd|bsd|aix|osf1|irix)[^a-z]/i) {
				$os_type = "Unix : $1"; }
			elsif (/x11|lynx|wget/i) { $os_type = "Unix"; }
			else { $os_type = ''; }

			if (/^Mozilla.+MSIE (\d+\.\d).+AOL ([\d\.]+)/i) { $agt_type = "AOL $2 (MSIE $1)"; } 
			elsif (/^Mozilla.+compatible.+MSIE ([\d\.]+)/) {
				$v = $1;
				$v =~ s/(\.\d)\d*/$1x/;
				$agt_type = "MS Internet Explorer $v";
				if ($os_type eq '') { $os_type = "Windows"; }
			}
			elsif (/^Opera/i) { $agt_type = "Opera"; }
			elsif (/^Mozilla\/([\d\.]+)/i && $_ !~ /compatible/i) { $v = $1; $v =~ s/(\.\d)\d*/$1x/; $agt_type = "Netscape $v"; }
			elsif (/WebTV/i) { $agt_type = "WebTV" }
			elsif (/Lynx/i)  { $agt_type = "Lynx" }
			elsif (/^Konqueror/i)  { $agt_type = "KDE Konqeror" }
			elsif (/^NetPositive/i)  { $agt_type = "NetPositive"; $os_type = "BeOS"; }
			elsif (/(Scooter|Slurp|Googlebot|ArchitextSpider|Arachnoidea|Ultraseek|Lycos_Spider|Gulliver|LinkWalker|WebCrawler|ip3000.com|SearchExpress|WebKangaroo|WebWombat|htdig)/i) { $agt_type = "Spider ($1 Spider)"; }
			elsif (/Spider|Robo/i) { $agt_type = "Spider"; }
			elsif (/^wget/i)  { $agt_type = "Wget" }
			else { $agt_type = 'unknown' }

			if ($os_type ne '') {
				$agt_type_long = "$agt_type ($os_type)";
			} else {
				$os_type = 'unknown';
				$agt_type_long = $agt_type;
			}

			$agent_type_long_hits{$agt_type_long} += $agent_hits_daily{$_};
			$agent_type_long_pview{$agt_type_long} += $agent_pages_views_daily{$_};
			$agent_type_long_visits{$agt_type_long} += $agent_visits_daily{$_};

			$agt_type =~ s/\..*//;
			$agent_type_hits{$agt_type} += $agent_hits_daily{$_};
			$agent_type_pview{$agt_type} += $agent_pages_views_daily{$_};
			$agent_type_visits{$agt_type} += $agent_visits_daily{$_};

			$os_type_hits{$os_type} += $agent_hits_daily{$_};
			$os_type_pview{$os_type} += $agent_pages_views_daily{$_};
			$os_type_visits{$os_type} += $agent_visits_daily{$_};
		}

	&mk_single_report('agents-a', $report_dir, 'g:bar',
		[ \@data_col0, \@data_col1, \@data_col3, \@data_col4 ]);

	&mk_file_type_report('agents-1', $report_dir, $params{'TopOfAgentT'} , $params{'TopOfAgentG'}, 'n, agents',
		\%agent_type_long_hits, '', \%agent_type_long_pview, \%agent_type_long_visits, $total_hits);

	&mk_file_type_report('agents-2', $report_dir,$params{'TopOfAgentT'} , $params{'TopOfAgentG'}, 'n, agents',
		\%agent_type_hits, '', \%agent_type_pview, \%agent_type_visits, $total_hits);

	&mk_file_type_report('agents-3', $report_dir, $params{'TopOfAgentT'} , $params{'TopOfAgentG'}, 'n, agents',
		\%os_type_hits, '', \%os_type_pview, \%os_type_visits, $total_hits);
} # end &mk_agents_report()


sub mk_errors_report {
	my ($report_dir) = @_;
	my ($i, $i404, $i403, $i500, $col1, $col2, $t);
	my (@files_error_404_n, @files_error_404_h, @files_error_403_n, 
		@files_error_403_h, @files_error_500_n, @files_error_500_h,
		@data_col0, @data_col1, @data_col0g);
	my (%errors, %errors_other);

# Error
	my @result = (sort { $files_error_daily{$b} <=> $files_error_daily{$a} } keys (%files_error_daily));
	foreach (@result) {
		($col1, $col2) = split(/\|/, $_);
		$errors{$col1} += $files_error_daily{$_};
		if ($col1 eq '404') { 
			$files_error_404_n[$i404] =  $col2;
			$files_error_404_h[$i404] =  $files_error_daily{$_};
			$i404++;
		}
		elsif ($col1 eq '403') { 
			$files_error_403_n[$i403] =  $col2;
			$files_error_403_h[$i403] =  $files_error_daily{$_};
			$i403++;
		}
		elsif ($col1 eq '500') { 
			$files_error_500_n[$i500] =  $col2;
			$files_error_500_h[$i500] =  $files_error_daily{$_};
			$i500++;
		} else {
			$t = $language{'code_error_'.$col1};
			$errors_other{$t} .= "$col2\t$files_error_daily{$_}\n";
		}
	}		

	if (@files_error_404_n > 1) {
		&mk_single_report('errors-404', $report_dir, 't:total, t:class',
			[ \@files_error_404_n, \@files_error_404_h ]); 
	}

	if (@files_error_403_n > 1) {
		&mk_single_report('errors-403', $report_dir, 't:total, t:class',
			[ \@files_error_403_n, \@files_error_403_h ]);
	}

	if (@files_error_500_n > 1) {
		&mk_single_report('errors-500', $report_dir, 't:total, t:class',
			[ \@files_error_500_n, \@files_error_500_h ]);
	}

	if ((keys %errors_other) > 1) {
		&mk_hash_sinple_report('errors-other', $report_dir, '', \%errors_other, '', 'errors.gif');
	}

	@result = (sort { $errors{$b} <=> $errors{$a} } keys (%errors));
	foreach (@result) {
			$data_col0[$i] = $language{'code_error_'.$_};
			$data_col0g[$i] = $_;
			$data_col1[$i] = $errors{$_};
			$i++;
	}
	&mk_single_report('errors', $report_dir, 'g:bar, t:total',
		[ \@data_col0, \@data_col1 ], 
 		[ \@data_col0g, \@data_col1 ]);
} # end &mk_errors_report()


sub mk_seeker_report {
	my ($report_dir) = @_;
	my ($total_hits, $nb, $r, $i, $word, $seeker_name, $expr);
	my (@r, @data1, @data2, @result);
	my (%seeker_cut, %seeker_nb, %seeker_expr, %seeker_word, %expr_all, %words_all, %w);

	if (! ($params{'MaxPerSeeker'} > 10)) { $params{'MaxPerSeeker'} = 10 }

	foreach(keys %seeker) {
		$nb = @r = reverse sort SubSortTxtByCol (split(/\n/, $seeker{$_}));
		if ($nb > $params{'MaxPerSeeker'}) {
			@r = @r[0..$params{'MaxPerSeeker'}];
		}
		$seeker_cut{$_} = join("\n", @r);
		$seeker_nb{$_}  = $nb;
		$total_hits += $nb;
	}

	&mk_file_type_report('seeker', $report_dir, $params{'TopOfSeekerT'} , $params{'TopOfSeekerG'}, 'n, refs',
		\%seeker_nb, '', '', '', $total_hits);

	&mk_hash_sinple_report('seeker-query', $report_dir, '', \%seeker_cut, '', 'seeker.gif');

	foreach(keys %seeker) {
		my $seeker_name = $_;
		my %w;
		foreach (split (/\n/, $seeker{$_})) {
			my ($word, $nb) = split(/\t/, $_);
			$seeker_expr{"$word|$seeker_name"} += $nb;
			$expr_all{$word} += $nb;
			$word =~ s/\+/ /g;
			$word =~ s/ +/ /g;
			$word =~ s/^ +| +$//g;
			$word =~ s/^-//g;
			foreach (split(/ +/, $word)) {
				$words_all{$_} += $nb;
				$w{$_} += $nb;
				$seeker_word{"$_|$seeker_name"} += $nb;
			}
		}
		$r = ""; $i = 0;
		foreach ((sort { $w{$b} <=> $w{$a} } keys (%w))) {
			if ((length($_) > 1) && (($i++ < $params{'MaxPerSeeker'}) || ($w{$_} >= 2))) {
				$r .= "$_\t$w{$_}\n";
			}
		}
		$seeker{$seeker_name} = $r;
	}
	&mk_hash_sinple_report('seeker-words', $report_dir, '', \%seeker, '', 'seeker.gif');

	%w =();
	@result = (sort { $expr_all{$b} <=> $expr_all{$a} } keys (%expr_all));
	$i = 0;
	foreach(@result) {
		if ($i <= 5) {
			$data1[$i] = $_;
			$data2[$i] = $expr_all{$_};
		}
		$i++;
		$w{$_} = $i;
	}

	$i = 0;
	%seeker = ();
	@result = (sort { $seeker_expr{$b} <=> $seeker_expr{$a} } keys (%seeker_expr));
	foreach(@result) {
		($expr, $seeker_name) = split(/\|/, $_);
		if ((length($expr) > 1) && (($i++ < $params{'MaxPerSeeker'}) || ($seeker_expr{$_} >= 5))) {
			$seeker{"$w{$expr} |$expr ($expr_all{$expr})"} .= "$seeker_name\t$seeker_expr{$_}\n";
		}
	}

	&mk_hash_sinple_report('seeker-query2', $report_dir, 'g:bar', \%seeker, [ \@data1, \@data2]);

	%w =();
	@result = (sort { $words_all{$b} <=> $words_all{$a} } keys (%words_all));
	$i = 0;
	foreach(@result) {
		if ($i <= 5) {
			$data1[$i] = $_;
			$data2[$i] = $words_all{$_};
		}
		$i++;
		$w{$_} = $i;
	}

	$i = 0;
	%seeker = ();
	@result = (sort { $seeker_word{$b} <=> $seeker_word{$a} } keys (%seeker_word));
	foreach(@result) {
		($word, $seeker_name) = split(/\|/, $_);
		if ((length($word) > 1) && (($i++ < $params{'MaxPerSeeker'}) || ($seeker_word{$_} >= 5))) {
			$seeker{"$w{$word} |$word ($words_all{$word})"} .= "$seeker_name\t$seeker_word{$_}\n";
		}
	}

	&mk_hash_sinple_report('seeker-words2', $report_dir, 'g:bar', \%seeker, [ \@data1, \@data2]);
} # end mk_seeker_report()


########################################
# Sub report types                     #
########################################

sub mk_day_type_report {
	my($id, $report_dir, $hash_data1, $hash_data2, $hash_data3, $hash_data4) = @_;
	my($i, @data_col0, @data_col1, @data_col2, @data_col3, @data_col4);

	$i = 0;
	foreach (sort(keys %$hash_data1)) {
		if (!(($_ eq '00') && ($$hash_data1{'00'} eq ''))) {
			$data_col0[$i] = $_;
			$data_col0[$i] =~ s/\d+ *\|//;
			$data_col1[$i] = $$hash_data1{$_};
			$data_col2[$i] = int ($$hash_data2{$_} / 1024);
			$data_col3[$i] = $$hash_data3{$_};
			$data_col4[$i] = $$hash_data4{$_};
			$i++;
		}
	}

	&mk_single_report($id, $report_dir, 't:total',
		[ \@data_col0, \@data_col1, \@data_col2,  \@data_col3, \@data_col4 ],
		[ \@data_col0, \@data_col4, \@data_col3 ], [ \@data_col0, \@data_col2, \@data_col1 ]);
}


sub mk_file_type_report {
	my($id, $report_dir, $max_result_txt, $max_result_grp, $do,
		$hash_data1, $hash_data2, $hash_data3, $hash_data4, $total) = @_;

	if (! ($max_result_txt) > 1) { $max_result_txt = 25 }
	if (! ($max_result_grp) > 1) { $max_result_grp = 5 }

	my ($nb, $i, $l, $k, $j, $do_a, $do_n, $do_l, $ko, $do_countrie, $do_refs, $do_agents, $do_paths, $nb_result, $max);
	my (@data1_col0t, @data1_col1t, @data1_col2t, @data1_col3t, @data1_col4t, @data1_col5t,
			@data1_col0g, @data1_col1g, @data1_col2g, @data1_col3g, @data1_col4g, @data1_col5g,
			@data2_col0t, @data2_col1t, @data2_col2t, @data2_col3t, @data2_col4t, @data2_col5t,
			@data2_col0g, @data2_col1g, @data2_col2g, @data2_col3g, @data2_col4g, @data2_col5g,
			@data3_col0t, @data3_col1t, @data3_col2t, @data3_col3t, @data3_col4t, @data3_col5t,
			@data3_col0g, @data3_col1g, @data3_col2g, @data3_col3g, @data3_col4g, @data3_col5g
		);
	if ($total <= 0) {
		warn "Warning &mk_file_type_report : total = $total\n";
		$total = 1;
	}

	$do =~ s/,/ /g;
	foreach(split (/ +/, lc($do))) {
		if ($_ eq 'a')    { $do_a = 1; }
		if ($_ eq 'n')    { $do_n = 1; }
		if ($_ eq 'l')    { $do_l = 1; }
		if (($_ eq 'countrie') || ($_ eq 'authusers')) { $do_countrie = 1; }
		if ($_ eq 'refs')     { $do_refs = 1; }
		if ($_ eq 'agents')   { $do_agents = 1; }
		if ($_ eq 'paths')    { $do_paths = 1; }
		if ($_ eq 'noKo')     { $ko = 0; } else { $ko = 1; }
	}

	my @result = (sort { $$hash_data1{$b} <=> $$hash_data1{$a} } keys (%$hash_data1));
	$nb_result = @result;
	for ($i = 0, $k = $nb_result - 1 ; (($i <= $nb_result -1) && ($i < 250)); $i++) {
		$_ = $result[$i];
		if ($$hash_data1{$_} > 0) {
			if ($do_n && ($i < $max_result_txt)) {
				$data2_col0t[$i] = $_;
				$data2_col1t[$i] = $$hash_data1{$_};
				$data2_col2t[$i] = $ko ? int ($$hash_data2{$_} / 1024) : $$hash_data2{$_};
				$data2_col3t[$i] = $$hash_data3{$_};
				$data2_col4t[$i] = $$hash_data4{$_};
				$data2_col5t[$i] = sprintf("%.2f", $$hash_data1{$_} * 100 / $total);
			}
			if ($do_a) {
				$data1_col0t[$i] = $_;
				$data1_col1t[$i] = $$hash_data1{$_};
				$data1_col2t[$i] = $ko ? int ($$hash_data2{$_} / 1024) : $$hash_data2{$_};
				$data1_col3t[$i] = $$hash_data3{$_};
				$data1_col4t[$i] = $$hash_data4{$_};
				$data1_col5t[$i] = sprintf("%.2f", $$hash_data1{$_} * 100 / $total);
			}
		}

		$_ = $result[$k--];
		if ($do_l && ($$hash_data1{$_} > 0) && ($l < $max_result_txt)) {
			$data3_col0t[$l] = $_;
			$data3_col1t[$l] = $$hash_data1{$_};
			$data3_col2t[$l] = $ko ? int ($$hash_data2{$_} / 1024) : $$hash_data2{$_};
			$data3_col3t[$l] = $$hash_data3{$_};
			$data3_col4t[$l] = $$hash_data4{$_};
			$data3_col5t[$l] = sprintf("%.2f", $$hash_data1{$_} * 100 / $total);
			$l++;
		}
	} # end foreach

	if (@data2_col0t < $max_result_grp)
		{ $max = @data2_col0t - 1; } else { $max = $max_result_grp; }
	@data2_col0g = @data2_col0t[0..$max];
	@data2_col1g = @data2_col1t[0..$max];

	if (@data3_col0t < $max_result_grp)
		{ $max = @data2_col0t - 1; } else { $max = $max_result_grp; }
	@data3_col0g = @data3_col0t[0..$max];
	@data3_col1g = @data3_col1t[0..$max];

	if ($do_countrie) {
		# All file
		if ($do_a) {
			&mk_single_report($id."-all", $report_dir, 't:total t:class',
				[ \@data1_col0t, \@data1_col1t, \@data1_col2t, \@data1_col3t, \@data1_col4t, \@data1_col5t ]);
		}
		# Top N report
		if ($do_n) {
			&mk_single_report($id."-n", $report_dir, 'g:bar t:class',
				[ \@data2_col0t, \@data2_col1t, \@data2_col2t, \@data2_col3t, \@data2_col4t, \@data2_col5t ],
				[ \@data2_col0g, \@data2_col1g]);
		}
		# Top N Least report
		if ($do_l) {
			&mk_single_report($id."-r", $report_dir, 'g:bar',
				[ \@data3_col0t, \@data3_col1t, \@data3_col2t, \@data3_col3t, \@data3_col4t, \@data3_col5t ],
				[ \@data3_col0g, \@data3_col1g ]);
		}
	} elsif ($do_agents) {
		&mk_single_report($id, $report_dir, 'g:bar t:class',
			[ \@data2_col0t, \@data2_col1t, \@data2_col3t, \@data2_col4t, \@data2_col5t ],
			[ \@data2_col0g, \@data2_col1g]);
	} elsif ($do_refs) {
		&mk_single_report($id, $report_dir, 'g:bar t:class',
			[ \@data2_col0t, \@data2_col1t, \@data2_col5t ],
			[ \@data2_col0g, \@data2_col1g ]);
	} elsif ($do_paths) {
		&mk_single_report($id, $report_dir, 't:class',
			[ \@data2_col0t, \@data2_col1t]);
	} else {
		# All file
		if ($do_a) {
			&mk_single_report($id."-all", $report_dir, 't:total t:class',
				[ \@data1_col0t, \@data1_col1t, \@data1_col2t, \@data1_col5t ]);
		}
		# Top N report
		if ($do_n) {
			&mk_single_report($id."-n", $report_dir, 'g:bar t:class',
				[ \@data2_col0t, \@data2_col1t, \@data2_col2t, \@data2_col5t ],
				[ \@data2_col0g, \@data2_col1g]);
		}
		# Top N Least report
		if ($do_l) {
			&mk_single_report($id."-r", $report_dir, 'g:bar',
				[ \@data3_col0t, \@data3_col1t, \@data3_col2t, \@data3_col5t ],
				[ \@data3_col0g, \@data3_col1g ]);
		}
	}
} # end &mk_file_type_report()


sub mk_hash_sinple_report {
	my($id, $report_dir, $options, $txt_data, $graph_data, $graph_img) = @_;
	#&mk_single_report('monthly', $report_dir, $options, \%hash
	#	[ \@graph_data_col1, \@graph_data_col2, \@graph_data_col3] #...
	#	);
	my %data = %$txt_data;

	print "hash_report $id\n" if ($DEBUG);
	my($tmp, @title);
	my $out_file = "$report_dir$id.html";
	$template_infos{'graph1'} = $template_infos{'graph2'} = '';

	$template_infos{'report_id'}       = $id;
	$template_infos{'title'}           = $language{$id."_title_page"};
	$template_infos{'title_head_help'} = $language{$id."_title_head_help"};
	$template_infos{'txt_intro'}       = $language{$id."_txt_intro"};
	$template_infos{'txt_help'}        = $language{$id."_txt_help"};
	if (@$graph_data > 1) {
		@title = split(/\s*,\s*/, $language{$id."_title_col_graph"});
		&create_graph($options, $language{$id."_title_head_graph"}, "$report_dir$id.gif", \@title, $graph_data);
		$template_infos{'graph1'} = &display_html_graph_border($language{$id."_title_head_graph"}, "$id.gif");
	} elsif ($graph_img ne '') {
		$template_infos{'graph1'} = &display_html_graph_border($language{$id."_title_head_graph"}, "$graph_img");
	}
	@title = split(/\s*,\s*/, $language{$id."_title_col_tab"});
	$template_infos{'tab'} = &diplay_html_tab_data_type2($language{$id."_title_head_tab"}, $options, \@title, \%data);
	$tmp = &repace_into_template_file($params{'TemplateStats'}, %template_infos);
	if (! ($tmp =~ s/FTLS_COPYRIGHT/$html_copyright/))
		{ die "You should not modifie or suppress Copyright, works stoped...\n"; }
	&save_txt_file($out_file, 'single report', $tmp);
	$add_to_menu{$id} = 1;
}


sub mk_single_report {
	my($id, $report_dir, $options, $txt_data, $graph_data, $graph_data2) = @_;
	#&mk_single_report('monthly', $report_dir, $options
	#	[ \@text_data_col1, \@text_data_col2, \@text_data_col3], #...
	#	[ \@graph_data_col1, \@graph_data_col2, \@graph_data_col3] #...
	#	);
	print "single_report $id\n" if ($DEBUG);
	my($tmp, @title);
	my $out_file = "$report_dir$id.html";
	$template_infos{'graph1'} = $template_infos{'graph2'} = '';

	my ($text_data_col1, $text_data_col2, $text_data_col3, $text_data_col4, $text_data_col5, $text_data_col6) = @$txt_data;

	$template_infos{'report_id'}       = $id;
	$template_infos{'title'}           = $language{$id."_title_page"};
	$template_infos{'title_head_help'} = $language{$id."_title_head_help"};
	$template_infos{'txt_intro'}       = $language{$id."_txt_intro"};
	$template_infos{'txt_help'}        = $language{$id."_txt_help"};
	if (@$graph_data > 1) {
		@title = split(/\s*,\s*/, $language{$id."_title_col_graph"});
		&create_graph($options, $language{$id."_title_head_graph"}, "$report_dir$id.gif", \@title, $graph_data);
		$template_infos{'graph1'} = &display_html_graph_border($language{$id."_title_head_graph"}, "$id.gif");
	}
	if (@$graph_data2 > 1) {
		@title = split(/\s*,\s*/, $language{$id."_title_col_graph2"});
		&create_graph($options, $language{$id."_title_head_graph2"}, $report_dir.$id."2.gif", \@title, $graph_data2);
		$template_infos{'graph2'} = &display_html_graph_border($language{$id."_title_head_graph2"}, $id."2.gif");
	}
	@title = split(/\s*,\s*/, $language{$id."_title_col_tab"});
	$template_infos{'tab'} = &diplay_html_tab_data_type1($language{$id."_title_head_tab"}, $options, \@title, $text_data_col1, $text_data_col2, $text_data_col3, $text_data_col4, $text_data_col5, $text_data_col6);
	$tmp = &repace_into_template_file($params{'TemplateStats'}, %template_infos);
	if (! ($tmp =~ s/FTLS_COPYRIGHT/$html_copyright/))
		{ die "You should not modifie or suppress Copyright, works stoped...\n"; }
	&save_txt_file($out_file, 'single report', $tmp);
	$add_to_menu{$id} = 1;
}

sub load_resume_file {
	my($file) = @_;
	print "      Load resume file: '$file'\n" if ($DEBUG);
	open (RESUMEFILE,"<$file") || die "  Error opening resume file: $file, error: $!\n";
	while(<RESUMEFILE>) {
		chomp($_);
		my @col = split(/\|/, $_);
		if (/^hourly/) {
			$col[1] = "$col[1]:00 - $col[1]:59";
			$all_hits_hourly{$col[1]} += $col[2]; 
			$all_bytes_hourly{$col[1]}  += $col[3]; 
			$all_pages_views_hourly{$col[1]} += $col[4]; 
			$all_visits_hourly{$col[1]} += $col[5];
		}
		elsif (/^daily/) {
			$all_hits_daily{$col[1]}  += $col[2]; 
			$all_bytes_daily{$col[1]} += $col[3]; 
			$all_pages_views_daily{$col[1]} += $col[4]; 
			$all_visits_daily{$col[1]}+= $col[5];
		}
		elsif (/^dayofweek/) {
			$all_hits_dayofweek{"$col[1]|$Week_days[$col[1]]"}   += $col[2];
			$all_bytes_dayofweek{"$col[1]|$Week_days[$col[1]]"}  += $col[3];
			$all_pages_views_dayofweek{"$col[1]|$Week_days[$col[1]]"}  += $col[4];
			$all_visits_dayofweek{"$col[1]|$Week_days[$col[1]]"} += $col[5];
		}
		elsif (/^monthly/) {
			$all_hits_monthly{$col[1]}  += $col[2];
			$all_bytes_monthly{$col[1]} += $col[3]; 
			$all_pages_views_monthly{$col[1]} += $col[4]; 
			$all_visits_monthly{$col[1]}+= $col[5];
		}
		elsif (/^year/) {
			$all_hits_year{$col[1]}  += $col[2]; 
			$all_bytes_year{$col[1]} += $col[3]; 
			$all_pages_views_year{$col[1]} += $col[4]; 
			$all_visits_year{$col[1]}+= $col[5];
		}
		elsif(/^files/) { 
			$files_hits_daily{$col[1]}  += $col[2]; 
			$files_bytes_daily{$col[1]} += $col[3];
			$files_visits_daily{$col[1]} += $col[5];
		}
		elsif(/^domains/) { 
			$domains_hits_daily{$col[1]}  += $col[2]; 
			$domains_bytes_daily{$col[1]} += $col[3]; 
			$domains_pages_views_daily{$col[1]} += $col[4]; 
			$domains_visits_daily{$col[1]}+= $col[5];
		}
		elsif(/^referrer/) {
			$referrer_hits_daily{"$col[1]|$col[2]"}  += $col[3]; 
			$referrer_pages_views_daily{"$col[1]|$col[2]"} += $col[5]; 
			$referrer_visits_daily{"$col[1]|$col[2]"} += $col[6];
		}
		elsif(/^agent/) {
			$agent_hits_daily{$col[1]}   += $col[2]; 
			$agent_pages_views_daily{$col[1]}  += $col[4];
			$agent_visits_daily{$col[1]} += $col[5];
		}
		elsif(/^error/) {
			$files_error_daily{"$col[1]|$col[2]"} += $col[3]; 
		}
		elsif(/^seeker/) {
			$seeker{$col[1]} .= "$col[2]\t$col[3]\n";
		}
		elsif(/^adverts/) {
			$adverts_hits_daily{"$col[1]|$col[2]"} += $col[3];
		}
		elsif(/^record/) {
			$record{$col[1]} = "$col[2]\t$col[3]";
		}
		elsif(/^trace_visit/) {
			$trace_visit{$col[1]} = $col[2];
		}
		elsif(/^users/) { 
			$users_hits_daily{$col[1]}  += $col[2]; 
			$users_bytes_daily{$col[1]} += $col[3]; 
			$users_pages_views_daily{$col[1]} += $col[4]; 
			$users_visits_daily{$col[1]}+= $col[5];
		}
	}
	close(RESUMEFILE);
}

sub init_resume_value {
	%trace_visit = ();
	%test_visit = ();
	%all_hits_year = ();
	%all_bytes_year = ();
	%all_pages_views_year = ();
	%all_visits_year = ();
	%all_hits_monthly = ();
	%all_bytes_monthly = ();
	%all_pages_views_monthly = ();
	%all_visits_monthly = ();
	%all_hits_daily = ();
	%all_bytes_daily = ();
	%all_pages_views_daily = ();
	%all_visits_daily = ();
	%all_hits_hourly = ();
	%all_bytes_hourly = ();
	%all_pages_views_hourly = ();
	%all_visits_hourly = ();
	%files_hits_daily = ();
	%files_bytes_daily = ();
	%files_visits_daily = ();
	%domains_hits_daily = ();
	%domains_bytes_daily = ();
	%domains_visits_daily = ();
	%domains_pages_views_daily = ();
	%users_hits_daily = ();
	%users_bytes_daily = ();
	%users_visits_daily = ();
	%users_pages_views_daily = ();
	%referrer_hits_daily = ();
	%referrer_visits_daily = ();
	%referrer_pages_views_daily = ();
	%agent_hits_daily = ();
	%agent_visits_daily = ();
	%agent_pages_views_daily = ();
	%files_error_daily = ();
	%adverts_hits_daily = ();
	%seeker = ();
	%record = ();
}

########################################
# Generate Graph and HTML subs         #
########################################

sub create_graph {
	my $options = lc(shift @_); # Line, Bar, Pie...
	my $graph_title = shift @_;
	my $img_file  = shift @_;
	my $legend   = shift @_;
	my $data = shift @_;

	my $ylabel = "$$legend[1] / $$legend[2] / $$legend[3]";
	$ylabel =~ s/\/\s*\///g;
	$ylabel =~ s/\s*\/\s*$//;
	$ylabel =~ s/^\s+|\s+$//;

if ($USE_GRAPH) {
	my $graph;
	undef $graph;
	if ($options =~ /pie/) { $graph = new GD::Graph::pie3d(); }
	elsif ($options =~ /bar/) { $graph = new GD::Graph::bars3d(); }
	else { $graph = new GD::Graph::lines3d(); }

	$graph->set( 
		long_ticks => 1,
		legend_placement => 'RT',  #RT
		y_plot_values => 1,
		x_labels_vertical => 1,  
		#x_label_position => 1/2,  # center
		bar_depth => 10,
		#fgclr => 'white', # cadrillage
		#boxclr => 'dblue', # fond
		accentclr => 'dblue',
		dclrs => [qw(lgreen lred cyan)],
		bar_spacing => 5,
		logo => "$params{'OutputDir'}/images/logo." . GD::Graph->export_format,
		logo_position => 'BR',

		transparent => 1,
		l_margin => 10,
		b_margin => 20,
		r_margin => 10,
		t_margin => 10,

		x_label => $$legend[0],
		y_label => $ylabel,
	   	y_min_value => 0,
		y_tick_number => 10,
		box_axis => 0,
		line_width => 3);
	if (($options !~ /pie/) && (@$legend > 2)) { 
		shift @$legend;
		$graph->set_legend(@$legend);
	}
	$graph->plot($data);
	open(OUT, ">$img_file") or die "Cannot open $img_file for write: $!";
	binmode OUT;
	print OUT $graph->gd->gif;
	close OUT;
}
}


sub display_html_graph_border {
	my $graph_title = shift @_;
	my $img_file  = shift @_;
	my $up = $template_infos{'img_up_dir'};
	if ($params{'Look'} eq "wf") {
return <<EOF;
<table width="98%" align="center" cellpadding="0" cellspacing="0" border="0">
<tr><td width="100%">
<TABLE BORDER=0 BGCOLOR='$params{'ReportTabBGColor'}' CELLSPACING="0" CELLPADDING="0" WIDTH='100%'>
<TR><TD><TABLE BORDER="0" CELLSPACING="1" CELLPADDING="2" WIDTH="100%">
<TR BGCOLOR="$params{'ReportTabHeadColor'}">
<TD COLSPAN="1">
<table cellpadding=0 cellspacing=0 border=0 width="100%"><tr>
	<td align=left><font size=3 face="Arial" color="$params{'ReportTabHeadTxtColor'}"><b>$graph_title</b></font></td>
	<td align='right'>&nbsp;</td>
	<td align=right><a href='#help'><img src='$up/images/$params{'ReportTabHelpIMG'}' alt='?' border=0></a></td>
</tr></table>
</TD></TR>
<TR BGCOLOR="$params{'ReportTabTD1Color'}"><TD BGCOLOR='$params{'ReportTabTD1Color'}'>
<TABLE BORDER=0 CELLSPACING=1 CELLPADDING=0 WIDTH='100%'>
<TR><TD VALIGN=TOP ALIGN=CENTER BGCOLOR="$params{'ReportTabTD1Color'}"><IMG SRC="$img_file" BORDER="0"></TD></TR>
</TABLE>
</TD></TR></TABLE>
</TD></TR></TABLE>
</TD></TR></TABLE>
<BR><BR>
EOF
	} else {
return <<EOF;
<TABLE BGCOLOR="$params{'ReportTabHeadColor'}" WIDTH="90%" BORDER="0" CELLPADDING="0" CELLSPACING="0">
<TR>
	<TD ROWSPAN="2" VALIGN="top" WIDTH="31"><IMG SRC="$up/images/tl-g.gif"></TD>
	<TD ROWSPAN="2" ALIGN="LEFT"><FONT FACE="Arial" COLOR="$params{'ReportTabHeadTxtColor'}"><B>$graph_title</B></A></TD>
	<TD VALIGN="top" ALIGN="RIGHT"><IMG SRC="$up/images/tr.gif""></TD>
	<TD BGCOLOR="$params{'ReportBGColor'}"><IMG SRC="$up/images/pix.gif"></TD>
</TR>
<TR><TD>&nbsp;</TD><TD VALIGN="top" ALIGN="RIGHT"><IMG SRC="$up/images/mr.gif"></TD></TR>
<TR><TD COLSPAN="4" WIDTH="100%">
<TABLE BORDER="0" CELLPADDING='1' CELLSPACING='2' BGCOLOR="$params{'ReportTabHeadColor'}" WIDTH="100%">
<TR><TD BGCOLOR="$params{'ReportTabBGColor'}" WIDTH="100%" ALIGN="CENTER">
<IMG SRC="$img_file" BORDER="0">
</TD></TR></TABLE>
</TD></TR></TABLE>
<BR><BR>
EOF
	}
}

sub diplay_html_tab_data_type1 {
	my ($tab_title, $options, $title, $tab1, $tab2, $tab3, $tab4, $tab5, $tab6) = @_;
	my ($add_class, $add_total);
	my $up = $template_infos{'img_up_dir'};
	if ($options =~ /class/) {  $add_class = 1; }
	if ($options =~ /total/) {  $add_total = 1; }
	my @tab1 = @$tab1;
	my @tab2 = @$tab2;
	my @tab3 = @$tab3;
	my @tab4 = @$tab4;
	my @tab5 = @$tab5;
	my @tab6 = @$tab6;
	my ($return, $i, $c, $nb_col, $nb_row, $place, $color);
	my ($max1, $max2, $max3, $max4, $max5, $max6);
	my ($total1, $total2, $total3, $total4, $total5, $total6);
	$total1 = $total2 = $total3 = $total4 = $total5 = $total6 = "&nbsp";
	$nb_col = @$title;
	if ($add_class == 1) {
		$nb_col++;
	}
if ($params{'Look'} eq "wf") {
	$return = <<EOF;
<TABLE width="98%" align="center" cellpadding="0" cellspacing="0" border="0">
<TR><TD width="100%">
<TABLE BORDER=0 BGCOLOR='$params{'ReportTabBGColor'}' CELLSPACING="0" CELLPADDING="0" WIDTH='100%'>
<TR><TD><TABLE BORDER=0 CELLSPACING=1 CELLPADDING=2 WIDTH='100%'>
<TR BGCOLOR="$params{'ReportTabHeadColor'}"><TD COLSPAN=$nb_col>
<TABLE cellpadding=0 cellspacing=0 border=0 width="100%">
<TR><td align=left><font size=3 face="Arial" color="$params{'ReportTabHeadTxtColor'}"><b>$tab_title</b></font></td>
	<TD>&nbsp;</TD>
	<TD align=right><A href='#help'><IMG src='$up/images/$params{'ReportTabHelpIMG'}' alt='? ' border=0></A></TD>
</TR></TABLE>
</TD></TR>
<TR  BGCOLOR="$params{'ReportTabTHColor'}">
EOF
	} else {
	$return = <<EOF;
<TABLE BGCOLOR="$params{'ReportTabHeadColor'}" WIDTH="90%" BORDER="0" CELLPADDING="0" CELLSPACING="0">
<TR>
	<TD ROWSPAN="2" VALIGN="top" WIDTH="31"><IMG SRC="$up/images/tl-t.gif"></TD>
	<TD ROWSPAN="2" ALIGN="LEFT"><FONT FACE="Arial" COLOR="$params{'ReportTabHeadTxtColor'}"><B>$tab_title</B></A></TD>
	<TD VALIGN="top" ALIGN="RIGHT"><IMG SRC="$up/images/tr.gif"></TD>
	<TD BGCOLOR="$params{'ReportBGColor'}"><IMG SRC="$up/images/pix.gif"></TD>
</TR>
<TR><TD>&nbsp;</TD><TD VALIGN="top" ALIGN="RIGHT"><IMG SRC="$up/images/mr.gif"></TD></TR>
<TR><TD COLSPAN="4" WIDTH="100%">
<TABLE BORDER="0" CELLPADDING='1' CELLSPACING='2' BGCOLOR="$params{'ReportTabHeadColor'}" WIDTH="100%">
<TR><TD BGCOLOR="$params{'ReportTabBGColor'}" WIDTH="100%">
<TABLE CELLPADDING='1' CELLSPACING='1' WIDTH="100%">
<TR BGCOLOR="$params{'ReportTabTHColor'}">
EOF
}
	if ($add_class == 1) {
		$return .= "<TD WIDTH=\"5%\">&nbsp;</TD>";
	}

	foreach (@$title) {
		$return .= "<TD><FONT SIZE=1 FACE=\"Arial, Hevetica\" COLOR=\"000000\"><CENTER><B>$_</B></CENTER></FONT></TD>";
	}
	$return .= "</TR>\n";

	foreach (@tab1) { s/\d+ *\|//; if ((/^\d+$/) && ($_ > $max1)) { $max1 = $_; } }
	foreach (@tab2) { s/\d+ *\|//; if ((/^\d+$/) && ($_ > $max2)) { $max2 = $_; } }
	foreach (@tab3) { s/\d+ *\|//; if ((/^\d+$/) && ($_ > $max3)) { $max3 = $_; } }
	foreach (@tab4) { s/\d+ *\|//; if ((/^\d+$/) && ($_ > $max4)) { $max4 = $_; } }
	foreach (@tab5) { s/\d+ *\|//; if ((/^\d+$/) && ($_ > $max5)) { $max5 = $_; } }
	foreach (@tab6) { s/\d+ *\|//; if ((/^\d+$/) && ($_ > $max6)) { $max6 = $_; } }

	$nb_col = @$title + 1;
	$nb_row = @$tab1 -1;
	for ($i = 0; $i <=  $nb_row; $i++) {
		if ($c) { $c = 0;  $return .= "<TR BGCOLOR=\"$params{'ReportTabTD2Color'}\">"; } else { $c = 1; $return .= "<TR BGCOLOR=\"$params{'ReportTabTD1Color'}\">"; }

		($tab1[$i], $total1) = &result_cell_present($tab1[$i], $max1, $$title[0], $total1);
		($tab2[$i], $total2) = &result_cell_present($tab2[$i], $max2, $$title[1], $total2);
		($tab3[$i], $total3) = &result_cell_present($tab3[$i], $max3, $$title[2], $total3);
		($tab4[$i], $total4) = &result_cell_present($tab4[$i], $max4, $$title[3], $total4);
		($tab5[$i], $total5) = &result_cell_present($tab5[$i], $max5, $$title[4], $total5);
		($tab6[$i], $total6) = &result_cell_present($tab6[$i], $max6, $$title[5], $total6);

		if ($add_class == 1) {
			$place = $i + 1;
			$return .= "<TD><FONT SIZE=1 FACE=\"Arial,Helvetica\" COLOR=\"000000\">$place</FONT></TD>";
		}

		if ($nb_col > 1)  { $return .= "<TD><FONT SIZE=1 FACE=\"Arial,Helvetica\" COLOR=\"000000\">$tab1[$i]</FONT></TD>"; }
		if ($nb_col > 2)  { $return .= "<TD><FONT SIZE=1 FACE=\"Arial,Helvetica\" COLOR=\"000000\">$tab2[$i]</FONT></TD>"; }
		if ($nb_col > 3)  { $return .= "<TD><FONT SIZE=1 FACE=\"Arial,Helvetica\" COLOR=\"000000\">$tab3[$i]</FONT></TD>"; }
		if ($nb_col > 4)  { $return .= "<TD><FONT SIZE=1 FACE=\"Arial,Helvetica\" COLOR=\"000000\">$tab4[$i]</FONT></TD>"; }
		if ($nb_col > 5)  { $return .= "<TD><FONT SIZE=1 FACE=\"Arial,Helvetica\" COLOR=\"000000\">$tab5[$i]</FONT></TD>"; }
		if ($nb_col > 6)  { $return .= "<TD><FONT SIZE=1 FACE=\"Arial,Helvetica\" COLOR=\"000000\">$tab6[$i]</FONT></TD>"; }

	$return .= "</TR>\n";
	}
	if ($add_total) {
		$return .= "<TR BGCOLOR=\"$params{'ReportTabTHColor'}\">";
		if ($add_class == 1) { $return .= "<TD WIDTH=\"5%\">&nbsp;</TD>"; }
		if ($nb_col > 1)  { 
			#if ($$title[0] =~ /%/) { if ($total1 > 95) { $total1 = 100; } $total1 .= "%"; } 
			$return .= "<TD><FONT SIZE=1 FACE=\"Arial,Helvetica\" COLOR=\"000000\"><B>Total :</B></FONT></TD>";
			}
		if ($nb_col > 2)  { 
			if ($$title[1] =~ /%/) { if ($total2 > 95) { $total2 = 100; } $total2 .= "%"; }
			$return .= "<TD><FONT SIZE=1 FACE=\"Arial,Helvetica\" COLOR=\"000000\"><CENTER><B>$total2</B></CENTER></FONT></TD>";
			}
		if ($nb_col > 3)  { 
			if ($$title[2] =~ /%/) { if ($total3 > 95) { $total3 = 100; } $total3 .= "%"; }
			$return .= "<TD><FONT SIZE=1 FACE=\"Arial,Helvetica\" COLOR=\"000000\"><CENTER><B>$total3</B></CENTER></FONT></TD>";
		}
		if ($nb_col > 4)  { 
			if ($$title[3] =~ /%/) { if ($total4 > 95) { $total4 = 100; } $total4 .= "%"; }
			$return .= "<TD><FONT SIZE=1 FACE=\"Arial,Helvetica\" COLOR=\"000000\"><CENTER><B>$total4</B></CENTER></FONT></TD>";
		}
		if ($nb_col > 5)  { 
			if ($$title[4] =~ /%/) { if ($total5 > 95) { $total5 = 100; } $total5 .= "%"; }
			$return .= "<TD><FONT SIZE=1 FACE=\"Arial,Helvetica\" COLOR=\"000000\"><CENTER><B>$total5</B></CENTER></FONT></TD>";
		}
		if ($nb_col > 6)  { 
			if ($$title[5] =~ /%/) { if ($total6 > 95) { $total6 = 100; } $total6 .= "%"; }
			$return .= "<TD><FONT SIZE=1 FACE=\"Arial,Helvetica\" COLOR=\"000000\"><CENTER><B>$total6</B></CENTER></FONT></TD>";
		}
	}
	$return .= <<EOF;
</TR></TABLE>
</TD></TR></TABLE>
</TD></TR></TABLE>
<BR><BR>
EOF
	return $return;
}

sub copyright_txt {
	return "<DIV ALIGN=\"CENTER\"><CENTER><FONT FACE=\"Arial\" SIZE=\"-2\"><EM>&copy Copyright 2000 <A HREF=\"http://www.ftls.org/\">FTLS</A> (Tyndiuk Fr&eacute;d&eacute;ric). All rights reserved.<BR>FTLS's CGI &amp; PHP Scripts Archive : <A HREF=\"http://www.ftls.org/cgi/\" TARGET=\"_top\">http://www.ftls.org/cgi/</A>.<BR>Report Generated by <A HREF=\"http://www.ftls.org/\" TARGET=\"_top\">e Web Log Analyzer</A></EM></FONT><BR><BR></CENTER></DIV>\n";
}

sub result_cell_present {
	my($cell_info, $max, $title_col, $total) = @_;
	if ($cell_info =~ /^[\d\.]+$/)  {
		$total += $cell_info;
		if ($max && ($cell_info == $max)) { $cell_info = "<B>$cell_info</B>"; }
		if ($title_col =~ /%/) { $cell_info = $cell_info."%"; }
		$cell_info = &commas($cell_info);
		$cell_info = "<CENTER>$cell_info</CENTER>";
	} else {
		if ($cell_info =~ /^\//) { $cell_info = $params{'ServerURL'}.$cell_info;  }
		if ($cell_info =~ /^https*:/) { $cell_info = "<A HREF=\"$cell_info\">$cell_info</A>"; }
	}
	return ($cell_info, $total);
}

sub diplay_html_tab_data_type2 {
	my ($tab_title, $options, $title, $hash_tab1) = @_;
	my %hash_tabx = %$hash_tab1;
	my $up = $template_infos{'img_up_dir'};

	my($return, $i, $c, $nb_col, $nb_row, $nb_sub, $color);
	my($max1, $max2, $max3, $max4, $max5, $max6);
	my($tab1, $tab2, $tab3, $tab4, $tab5, $tab6, $tab7, $tab8, @stab, @result);
	$nb_col = @$title;

if ($params{'Look'} eq "wf") {
	$return = <<EOF;
<TABLE width="98%" align="center" cellpadding="0" cellspacing="0" border="0">
<TR><TD width="100%">
<TABLE BORDER=0 BGCOLOR='$params{'ReportTabBGColor'}' CELLSPACING="0" CELLPADDING="0" WIDTH='100%'>
<TR><TD><TABLE BORDER=0 CELLSPACING=1 CELLPADDING=2 WIDTH='100%'>
<TR BGCOLOR="$params{'ReportTabHeadColor'}">
<TD COLSPAN=$nb_col>
<TABLE cellpadding=0 cellspacing=0 border=0 width="100%">
<TR><td align=left><font size=3 face="Arial" color="$params{'ReportTabHeadTxtColor'}"><b>$tab_title</b></font></td>
	<TD>&nbsp;</TD>
	<TD align=right><A href='#help'><IMG src='$up/images/$params{'ReportTabHelpIMG'}' alt='? ' border=0></A></TD>
</TR></TABLE>
</TD></TR>
<TR BGCOLOR="$params{'ReportTabTHColor'}">
EOF
} else {
	$return = <<EOF;
<TABLE BGCOLOR="$params{'ReportTabHeadColor'}" WIDTH="90%" BORDER="0" CELLPADDING="0" CELLSPACING="0">
<TR>
	<TD ROWSPAN="2" VALIGN="top" WIDTH="31"><IMG SRC="$up/images/tl-t.gif"></TD>
	<TD ROWSPAN="2" ALIGN="LEFT"><FONT FACE="Arial" COLOR="$params{'ReportTabHeadTxtColor'}"><B>$tab_title</B></A></TD>
	<TD VALIGN="top" ALIGN="RIGHT"><IMG SRC="$up/images/tr.gif"></TD>
	<TD BGCOLOR="$params{'ReportBGColor'}"><IMG SRC="$up/images/pix.gif"></TD>
</TR>
<TR><TD>&nbsp;</TD><TD VALIGN="top" ALIGN="RIGHT"><IMG SRC="$up/images/mr.gif"></TD></TR>
<TR><TD COLSPAN="4" WIDTH="100%">
<TABLE BORDER="0" CELLPADDING='1' CELLSPACING='2' BGCOLOR="$params{'ReportTabHeadColor'}" WIDTH="100%">
<TR><TD BGCOLOR="$params{'ReportTabBGColor'}" WIDTH="100%">
<TABLE CELLPADDING='1' CELLSPACING='1' WIDTH="100%">
<TR BGCOLOR="$params{'ReportTabTHColor'}">
EOF
}
	foreach (@$title) {
		$return .= "<TD ALIGN=\"CENTER\"><FONT SIZE=1 FACE=\"Arial, Helvetica\" COLOR=\"000000\"><B>$_</B></FONT></TD>";
	}
	$return .= "</TR>\n";

	$nb_col = @$title + 1 - 1;
	if ((keys %hash_tabx)[1] =~ /\|/) {
		@result = sort { my $ab = "$a\t$b"; $ab =~ s/(\d+) *\|.*\t(\d+) *\|.*//;  $1 <=> $2; } keys(%hash_tabx);
	} else {
		@result = sort keys(%hash_tabx);
	}
	foreach (@result) {
		$tab1 = $_;
		$tab1 =~ s/\d+ *\|//;
		@stab = split(/\n/, $hash_tabx{$_});
		$nb_sub = @stab;
		$tab1 = &result_cell_present_type2($tab1);
		$a .= "<TD ROWSPAN=\"$nb_sub\" VALIGN=\"TOP\"><FONT SIZE=1 FACE=\"Arial,Helvetica\" COLOR=\"000000\"><B>$tab1</B></FONT></TD>";
		if ($c) { $c = 0;  $color = $params{'ReportTabTD2Color'}; } else { $c = 1; $color = $params{'ReportTabTD1Color'}; }
		foreach(@stab) {
			$return .= "<TR BGCOLOR=\"$color\">";
			($tab2, $tab3, $tab4, $tab5, $tab6, $tab7) = split(/\t/, $_);
			if ($a ne '') { $return .= $a; $a = ''}
			$tab2 = &result_cell_present_type2($tab2);
			$tab3 = &result_cell_present_type2($tab3);
			if ($nb_col > 1)  { $return .= "<TD><FONT SIZE=1 FACE=\"Arial,Helvetica\" COLOR=\"000000\">$tab2</FONT></TD>"; }
			if ($nb_col > 2)  { $return .= "<TD><FONT SIZE=1 FACE=\"Arial,Helvetica\" COLOR=\"000000\">$tab3</FONT></TD>"; }
			if ($nb_col > 3)  { $return .= "<TD><FONT SIZE=1 FACE=\"Arial,Helvetica\" COLOR=\"000000\">$tab4</FONT></TD>"; }
			if ($nb_col > 4)  { $return .= "<TD><FONT SIZE=1 FACE=\"Arial,Helvetica\" COLOR=\"000000\">$tab5</FONT></TD>"; }
			if ($nb_col > 5)  { $return .= "<TD><FONT SIZE=1 FACE=\"Arial,Helvetica\" COLOR=\"000000\">$tab6</FONT></TD>"; }
			if ($nb_col > 6)  { $return .= "<TD><FONT SIZE=1 FACE=\"Arial,Helvetica\" COLOR=\"000000\">$tab7</FONT></TD>"; }
			if ($nb_col > 7)  { $return .= "<TD><FONT SIZE=1 FACE=\"Arial,Helvetica\" COLOR=\"000000\">$tab8</FONT></TD>"; }
			$return .= "</TR>\n";
		}
	}
	$return .= <<EOF;
</TR></TABLE>
</TD></TR></TABLE>
</TD></TR></TABLE>
<BR><BR>
EOF
	return $return;
}


sub result_cell_present_type2 {
	my($cell_info) = @_;
	if ($cell_info !~ /^[\d\.]+$/)  {
		if ($cell_info =~ /^\//) { $cell_info = $params{'ServerURL'}.$cell_info;  }
		if ($cell_info =~ /^https*:/) { $cell_info = "<A HREF=\"$cell_info\">$cell_info</A>"; }
	}
	return $cell_info;
}


########################################
# Litle subs                           #
########################################

sub loadcopyright {
	my $c = &copyright_txt();
	if (length $c <=> int(666*0.666 -19)) { die "You should not modifie or suppress Copyright, works stoped...\n"; }
	return $c;
}

sub last_updated_file {
	return (stat($_[0]))[10];
}

sub commas {
	local($_) = @_;
	1 while s/(.*\d)(\d\d\d)/$1.$2/;
	$_;
}

sub SubSortTxtByCol {
	my ($c10, $c11) = split (/\t/, $a);
	my ($c20, $c21) = split (/\t/, $b);
	$c11 <=> $c21;
}

sub conv_date {
	my($time, $format) = @_;
	if (! $time)   { $time = time; }
	if (! $format) { $format = '%D' }
	my($sec, $min, $hour, $mday, $mon, $year, $wday, $yday, $isdst) = localtime($time);
	my $full_en_text_date = localtime($time);
	$mon++;
	my $yearl = $year + 1900;
	if ($year > 99) { $year -= 100; }
	my $short_mon  = substr($Months[$mon - 1],  0, 3);
	my $short_wday = substr($Week_days[$wday], 0, 3);
	my $mdayl = sprintf('%02d', $mday);
	my $monl  = sprintf('%02d', $mon);
	$sec   = sprintf('%02d', $sec);
	$min   = sprintf('%02d', $min);
	$hour  = sprintf('%02d', $hour);
	$year  = sprintf('%02d', $year);

	$_ = $format;
	s/%%/%/g;
	s/%a/$short_wday/g;
	s/%A/$Week_days[$wday]/g;
	s/%b|%h/$short_mon/g;
	s/%B/$Months[$mon-1]/g;
	s/%c/$full_en_text_date/g;
	s/%d/$mdayl/g;
	s/%D|%x/$monl\/$mday\/$year/g;
	s/%e/$mday/g;
	s/%m/$monl/g;
	s/%H/$hour/g;
	s/%j/$yday/g;
	s/%M/$min/g;
	s/%S/$sec/g;
	s/%n/\n/g;
	s/%t/\t/g;
	s/%X|%T/$hour:$min:$sec/g;
	s/%y/$year/g;
	s/%Y/$yearl/g;
	return $_;
}

sub calendar {
	my ($day, $month, $year, $report_dir_base, $report_type) = @_;
	my ($up, $date, $return, $day_txt);
	my $month_txt = $month;
	$month++; $month--;
	if ($report_type eq 'd') { $up = '..' }
	if ($report_type eq 'm') { $up = '.' }
	
	my ($start_day, $days_in_month) = &PerpetualCalendar($month,1,$year);

	my $curr_day = 0;
	$return  = "<table border=0 cellspacing=1 cellpadding=0 bgcolor=\"#FFFFFF\" align=\"center\" bordercolor=\"#FFFFFF\">";
	#$return .= "<TR BGCOLOR=\"$params{'CalTHColor'}\"><TD COLSPAN=7 VALIGN=TOP ALIGN=CENTER><FONT FACE=\"Arial\" SIZE=\"-1\"><B>$Months[$month-1] $year</B></FONT></TD></TR>";
	$return .= "<TR BGCOLOR=\"$params{'CalTHColor'}\">";
	foreach (@Week_days_short) {
		$return .= "<TD width=16 ALIGN=\"CENTER\"><FONT FACE=\"Arial, Helvetica\" SIZE=\"1\" COLOR=\"$params{'CalTHTxtColor'}\"> $_ </font></TD>";
	}
	$return .= "</TR>\n";

	foreach $date (1-$start_day .. $days_in_month) {
		if ($curr_day == 0) { $return .= "<TR BGCOLOR=\"$params{'CalTDColor'}\">"; }
		if ($date > 0) {
			if ($date < 10) { $day_txt = "0$date" } else { $day_txt = $date; }
			if ((-d $report_dir_base."$year/$month_txt/$day_txt/")  && ($date != $day)) {
				$date = "<A HREF=\"$up/$day_txt/index.html\" TARGET=\"_top\">$date</A>";
			}
			if ($date == $day) { $date = "<STRONG><I>$date<I></STRONG>"; }
			$return .= "<TD WIDTH=\"16\" ALIGN=\"CENTER\"><FONT FACE=\"Arial, Helvetica\" SIZE=\"1\" COLOR=\"$params{'CalTDTxtColor'}\">$date</font></TD>";
		} else {
			$return .= "<TD><FONT FACE=\"Arial, Helvetica\" SIZE=\"1\" COLOR=\"$params{'CalTDTxtColor'}\">&nbsp;</FONT></TD>";
		}
		$curr_day++;
		if ($curr_day == 7) {
			$return .= "</TR>\n";
			$curr_day=0;
		}
	}
	return  $return ."</TABLE>\n";
} #end of &calendar()


sub PerpetualCalendar {
	# This perpetual calendar routine provides accurate day/date
	# correspondences for dates from 1601 to 2899 A.D.  It is based on
	# the Gregorian calendar, so be aware that early correspondences
	# may not always be historically accurate.  The Gregorian calendar
	# was adopted by the Italian states, Portugal and Spain in 1582,
	# and by the Catholic German states in 1583.  However, it was not
	# adopted by the Protestant German states until 1699, by England
	# and its colonies until 1752, by Sweden until 1753, by Japan
	# until 1873, by China until 1912, by the Soviet Union until 1918,
	# and by Greece until 1923.
	my ($perp_mon, $perp_day, $perp_year) = @_;
	my ($key, $perp_dow, $perp_eom, $perp_days, $perp_sofar, $perp_togo);
	my %day_counts = (1,0,2,31,3,59,4,90,5,120,6,151,7,181, 8,212,9,243,10,273,11,304,12,334);
	$perp_days = (($perp_year-1601)*365)+(int(($perp_year-1601)/4));
	$perp_days += $day_counts{$perp_mon};
	$perp_days += $perp_day;
	$perp_sofar = $day_counts{$perp_mon};
	$perp_sofar += $perp_day;
	$perp_togo = 365-$perp_sofar;
	if (int(($perp_year-1600)/4) eq (($perp_year-1600)/4)) {
		$perp_togo++;
		if ($perp_mon > 2) {
			$perp_days++;
			$perp_sofar++;
			$perp_togo -= 1;
		}
	}
	foreach $key (1700,1800,1900,2100,2200,2300,2500,2600,2700) {
		if ((($perp_year == $key) && ($perp_mon > 2))
		  || ($perp_year > $key)) {
			$perp_days -= 1;
		}
	}
	$perp_dow = $perp_days - (int($perp_days/7)*7);
	if ($perp_dow == 7) { $perp_dow = 0; }
	$perp_eom = 31;
	if (($perp_mon == 4) || ($perp_mon == 6)
	  || ($perp_mon == 9) || ($perp_mon == 11)) {
		$perp_eom = 30;
	}
	if (($perp_mon == 2)) {
		$perp_eom = 28;
	}
	if ((int(($perp_year-1600)/4) eq (($perp_year-1600)/4))
	  && ($perp_mon == 2)) {
		$perp_eom = 29;
	}
	foreach $key (1700,1800,1900,2100,2200,2300,2500,2600,2700) {
		if ($perp_year == $key) {
			if ($perp_mon == 1) {
				$perp_togo -= 1;
			}
			elsif ($perp_mon == 2) {
				$perp_togo -= 1;
				$perp_eom = 28;
			}
			else {
				$perp_sofar -= 1;
			}
		}
	}
	return ($perp_dow, $perp_eom, $perp_days, $perp_sofar, $perp_togo);
}
